import { PropsWithChildren } from 'react';
import { BackgroundBlurLevel, BackgroundFilter } from '@stream-io/video-filters-web';
export type BackgroundFiltersProps = {
    /**
     * A list of URLs to use as background images.
     */
    backgroundImages?: string[];
    /**
     * The background filter to apply to the video (by default).
     * @default undefined no filter applied
     */
    backgroundFilter?: BackgroundFilter;
    /**
     * The URL of the image to use as the background (by default).
     */
    backgroundImage?: string;
    /**
     * The level of blur to apply to the background (by default).
     * @default 'high'.
     */
    backgroundBlurLevel?: BackgroundBlurLevel;
    /**
     * The base path for the TensorFlow Lite files.
     * @default 'https://unpkg.com/@stream-io/video-filters-web/tf'.
     */
    basePath?: string;
    /**
     * The path to the TensorFlow Lite WebAssembly file.
     *
     * Override this prop to use a custom path to the TensorFlow Lite WebAssembly file
     * (e.g., if you choose to host it yourself).
     */
    tfFilePath?: string;
    /**
     * The path to the TensorFlow Lite model file.
     * Override this prop to use a custom path to the TensorFlow Lite model file
     * (e.g., if you choose to host it yourself).
     */
    modelFilePath?: string;
    /**
     * When a started filter encounters an error, this callback will be executed.
     * The default behavior (not overridable) is unregistering a failed filter.
     * Use this callback to display UI error message, disable the corresponsing stream,
     * or to try registering the filter again.
     */
    onError?: (error: any) => void;
};
export type BackgroundFiltersAPI = {
    /**
     * Whether the current platform supports the background filters.
     */
    isSupported: boolean;
    /**
     * Indicates whether the background filters engine is loaded and ready.
     */
    isReady: boolean;
    /**
     * Disables all background filters applied to the video.
     */
    disableBackgroundFilter: () => void;
    /**
     * Applies a background blur filter to the video.
     *
     * @param blurLevel the level of blur to apply to the background.
     */
    applyBackgroundBlurFilter: (blurLevel: BackgroundBlurLevel) => void;
    /**
     * Applies a background image filter to the video.
     *
     * @param imageUrl the URL of the image to use as the background.
     */
    applyBackgroundImageFilter: (imageUrl: string) => void;
};
/**
 * The context value for the background filters context.
 */
export type BackgroundFiltersContextValue = BackgroundFiltersProps & BackgroundFiltersAPI;
/**
 * A hook to access the background filters context API.
 */
export declare const useBackgroundFilters: () => BackgroundFiltersContextValue;
/**
 * A provider component that enables the use of background filters in your app.
 *
 * Please make sure you have the `@stream-io/video-filters-web` package installed
 * in your project before using this component.
 */
export declare const BackgroundFiltersProvider: (props: PropsWithChildren<BackgroundFiltersProps>) => import("react/jsx-runtime").JSX.Element;
